@@ -6,7 +6,7 @@ Kafka::Consumer - Perl interface for Kafka consumer client.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Consumer> version 0.9012 .
+This documentation refers to C<Kafka::Consumer> version 1.001001 .
 
 =cut
 
@@ -18,7 +18,7 @@ use warnings;
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '0.9012';
+our $VERSION = '1.001001';
 
 #-- load the modules -----------------------------------------------------------
 
@@ -161,7 +161,7 @@ Provides Kafka FETCH and OFFSETS requests.
 
 =item *
 
-Supports parsing the Apache Kafka 0.9 Wire Format protocol.
+Supports parsing the Apache Kafka 0.9+ Wire Format protocol.
 
 =item *
 
